/*
**		tforce.cpp
**
**		Pulls the cursor around the screen in a "random" fashion.
**		Uses the sum of six sine waves. 
**
**              Keith Jolley & Scott makeig, NHRC
**
*/
struct	Vector  BuffetForce(double t, struct Vector tPoint, char practice);
int		Sign(double test);
/************************************************************************
**	function Buffetforce
**	Given a time, t, calculates an "attracting force."
**	Frequencies, Ao, Bo, ..., should be non-multiples
**	of each other for best effects. Currently using random()...
**	Ax, Ay, ..., are the amplitudes in that plane of
**	the forcing function.
*************************************************************************/
struct	Vector  BuffetForce(double t, struct Vector tPoint, char practice)
{
	int i;
	struct Vector tempV;

	double	theta;			/* angle of buffet force to horizontal		*/
	double	force;			/* magnitude of the force					*/
	double	r;				/* distance of target from (0,0)       		*/

	/*
	** Find the distance from the center to the target.
	*/
	r = hypot(tPoint.x, tPoint.y);

	if (practice) {
		t += t;
		force = -innerSlope * r;
	}

	else if (r < innerRadius) {
		/*
		** In the center of the screen, there is a self small
		** repelling force, at the edges, there is a gutter.
		*/
		force = innerSlope * r * r;
	}
	else{
		/*
		** Send the target to the gutter.
		*/
		force =(outerSlope * (r - gutterRadius));
	}
	/*
	** Angle of the force vector.
	** The standard atan2 does not crash given (0,0).
	** It crashes using Borland C++... :(
	*/
	if (abs(tPoint.x) < 0.0001)
		theta = atan2(tPoint.y, Sign(tPoint.x)*0.0001);
	else
		theta = atan2(tPoint.y, tPoint.x);
	/*
	** Sum of sines buffetting force.
	*/
	tempV.x = tempV.y = 0.0;
	for (i=0;i<FREQS;i++) {
		tempV.x  += A[i].x*cos(w[i].x*t + phi[i].x);
		tempV.y  += A[i].y*sin(w[i].y*t + phi[i].y);
	}
	/*
	** Scale to screen and a constant
	** scale factor.
	*/
	tempV.x *= bfScale * normalAmp.x;
	tempV.y *= bfScale * normalAmp.y;

	/*
	** direction of the total buffet force
	*/
	tempV.x += force * cos(theta);
	tempV.y += force * sin(theta);

	if (practice) {/* keep the target near the center	*/
		tempV.x /= 250.0;
		tempV.y /= 250.0;
	}
	return(tempV);
}
/********************************************************************/
int	Sign(double test){

	if (test >= 0)
		test = 1;
	else
		test = -1;
	return test;
}
