/******************************************************************
**       tinit.cpp - initialize the COMPTRACK vigilance test program
**
**       Keith Jolley & Scott Makeig, NHRC (1996)
**
**       Reference: NHRC Technical Document 96-3C
**       Naval Health REsearch Center, San Diego CA 92186-5122
**
** NOTE: To adjust task difficulty/speed, adjust BUFFETING_FORCE below.
** NOTE: To adjust trackball force, adjust INPUT_FORCE below.
**
** To compile:  Use Borland C++ (3.1-). See Makefile.
** 
******************************************************************/
#define BUFFETING_FORCE 0.44
#define INPUT_FORCE    250.0
/*****************************************************************/
#include <time.h>
#include <stdio.h>
#include <stdlib.h>
#include <signal.h>
#include <graphics.h>
#include <conio.h>
#include <math.h>
#include <sys\stat.h>
#include <string.h>
#include <fcntl.h>
#include <io.h>
#include <dos.h>
#include <bios.h>
#include <graphics.h>

#undef  DEBUG
#define pi M_PI
#define CLIPOFF 1 		/* see also fill_style in TDRAW.cpp\DrawTarget	*/
						/* and TMAIN.cpp\main	putimage				*/

#define FREQS			6				/* number of freqs of buffet	*/
#define Xra				1.618034		/* golden ratio					*/
#define Yra				1.5707963		/* pi/2							*/
#define bgColor			BLACK			/* background color				*/
#define BullsEyeColor	LIGHTGRAY		/* color of inner bullseye ring	*/
#define	TargetColor		WHITE   		/* target point color			*/

#define targetRadius	10				/* hard code these later 2 speed*/
#define innerBullRadius	targetRadius	/* radius of inner bullseye 	*/
#define outerBullRadius targetRadius*2	/* radius of outer bullseye		*/
#define	outerSlope 		-1.5			/* slope of virtual hill		*/
#define innerSlope		0.50
#define	gutterRadius	20 * targetRadius	/* radius of gutter			*/
#define innerRadius		6 * targetRadius	/* radius of 'flat' portion	*/
											/* of screen where target 	*/
											/* is not pulled to gutter.	*/
/*
**                      ___________
**        			/ /|           |\        outer
** \     target--> 0/  |           |  \ <--  slope --> /
**   \            /    |--inrRad---|    \            /
**     \        /                         \        /
**       \    /                             \    /
**         \/                                 \/
**          |<-------- gutterRadius --------->|
*/
#define DISK_WRITE
#define WRITE_ERROR		FALSE
#define WRITE_OK		TRUE
#define	ESC_KEY			27
#define	CHARS_PER_LINE	25
#define PORT			888 /* port 0x3BC in decimal */
#define LOWER_SIGNAL	'z'
#define RAISE_SIGNAL	0x00
#undef  SIG_LENGTH_NOT 	2
#define TRUE			99
#define FALSE			0
#define SETTINGS 		( 0x80 | 0x02 | 0x00 | 0x00 )

/*
** global data types
*/
	struct Vector {
		double	x;
		double	y;
	};
	struct iVector {
		int 	x;
		int 	y;
	};
	struct outputStruct {
		struct 	iVector	cPoint;
		struct 	Vector	tPoint;
        struct	Vector	fPoint;
		};
/*
**	global data
*/
	FILE 	*file;
	char	buf[CHARS_PER_LINE];
	char	outputString[CHARS_PER_LINE];

/*
**	amplitude, frequency, and phase angle
*/
	struct Vector A[FREQS];
	struct Vector w[FREQS];
	struct Vector phi[FREQS];
/*
**	normal sums the amplitudes and normalizes to the
**	size of the screen rect so that the forcing function
**	will not go off of the screen.
*/
	struct 	Vector	normalAmp;

/*
**	offset translates the origin of the drawing coordinates
**	to the center of the screen.
*/
	struct	Vector	offset;
	struct	Vector	tOffset;

/*
**	pseudo-physical constants
*/
        double  dt  =   0.015;  /*      time step
								natural frequency: k/m
								damping ratio: sqrt(c*c-4mk)
								probobly best if over or
								critically damped					*/

        double  m = 1.00;               /*      mass of target                                          */
        double  c = 3.00;               /*      screen friction constant                        */
        double  k = 0.50;               /*      users spring constant. A higher
                                                k value gives the user stronger
                                                pull on the target.                                     */

        double  bfScale =  BUFFETING_FORCE;/* scale the buffeting force    */
        double  inScale =  INPUT_FORCE;    /* scale the users input force  */

/*
**	these are defined globally to speed processing.
*/
	double	sqDt = 	dt*dt;
	double	divisor = (m + c*dt + k*sqDt);

#include "c:\track\src\tfollow.cpp"
#include "c:\track\src\tforce.cpp"
#include "c:\track\src\tvideo.cpp"
#include "c:\track\src\tdraw.cpp"
#include "c:\track\src\tmouse.cpp"
#include "c:\track\src\toutput.cpp"
#include "c:\track\src\tmain.cpp"

/*
** prototypes
*/
int 	MyRand(void);
void	InitGlobals(void);
int 	InitGraphics(void);
void 	ShutDown(void);
void 	PanicStop(void);

/*******************************************************************/
void	InitGlobals(void){
	int i;

	offset.x = getmaxx() / 2;
	offset.y = getmaxy() / 2;
	tOffset.x = offset.x - 2*targetRadius;
	tOffset.y = offset.y - 2*targetRadius;

	normalAmp.x = 0.0;
	normalAmp.y = 0.0;

	for (i=0;i<FREQS;i++) {
		w[i].y = pow(Xra, i);
		w[i].x = pow(Yra, i);
		A[i].x = 1.0 / w[i].x;
		A[i].y = 1.0 / w[i].y;
		phi[i].x = (2.0 * pi * random(1024))/1024;
		phi[i].y = (2.0 * pi * random(1024))/1024;
		normalAmp.x = A[i].x;
		normalAmp.y = A[i].y;
	}
	normalAmp.x = offset.x / normalAmp.x;
	normalAmp.y = offset.y / normalAmp.y;

	outportb(PORT, LOWER_SIGNAL);	/* clear the port			*/
	strcpy(outputString, "\0");
}
/*****************************************************************
**
**	This routine 'should' be the only line that needs to be changed
**	for other c environments. Need a two page grahics mode.
**
*****************************************************************/
int 	InitGraphics(void){

		int n, gdriver = VGA, gmode = VGAMED, gerr;

		initgraph(&gdriver, &gmode, "c:\\borlandc\\bgi");
		gerr = graphresult();
		if (gerr != grOk)
			initgraph(&gdriver, &gmode, ".\\");
		gerr = graphresult();
		if (gerr != grOk) {
			printf("BGI error: %s/n", grapherrormsg(gerr));
			exit(gerr);
		}
		else{
			for (n=0;n<=1;n++) {
				setvisualpage(n);
				setactivepage(n);
				cleardevice();
				setviewport(0, 0, getmaxx(), getmaxy(), CLIPOFF);
				setbkcolor(bgColor);
			}
		}
		setvisualpage(1);
		setactivepage(1);
		return gerr;	/* will always return grOk				*/
}
/****************************************************************/
void ShutDown(void) {

        DeInitMouse;    /* take out trap, computer will crash else  */
	cleardevice();  /* shake the etch-a -sketch...				*/
	closegraph();	/* put display back to text mode			*/

	return;
}
/****************************************************************/
 void PanicStop(void) {
	ShutDown();
	printf("DISK WRITE ERROR, DATA NOT WRITTEN...\n");
	printf("ENSURE THAT THE C: DRIVE HAS SPACE AVAILABLE\n");
	printf("PROGRAM QUITTING.\n");

	exit(1);
}
